// -- File: STACK.CC

CLASS Stack OF Array()
/* ----------------------------------------------------------------------
    Description:
 
       This custom class is a sub-class of the built-in Visual dBASE 
       Array class. The original methods were written for dBASE IV by 
       Frank A. Deviney, Jr., but were completely rewritten for Visual 
       dBASE. (A couple of Frank's routines were released as 
       unnecessary)
 
       A stack is a single-column array. Elements are added by 
       "pushing" them, they are removed by "popping" them. 
 
       Note: Because a stack is a subclass of array, all properties 
       and methods of arrays (size, scan, etc.) are available. Use 
       caution in using such methods or in addressing an element by 
       its number, as in oMyStack[n]. These methods and adressing are
       not useable for a stack as such, but are only available because
       of its implementation as an array. (Jay Parsons)
 
    Programmers: 
 
       Ken Mayer
 
    History: 
 
       11/27/1996 -- original version of these routines as a class.
 
    Instantiation: 
 
       set procedure to stack.cc additive
       oMyStack = new Stack()
 
    Methods:
       More details on each of these is contained in the commentary for
       each method ...
    
       Release()
         Releases this object (and others) from memory
  
       Push()
         Add element to stack
 
       Pop()
         Remove element from stack
 
       GetTop()
         Return top element, but does not remove it
  
       Empty()
         Returns logical .t./.f. depending on whether or not
         the stack is empty.
 
    Properties:
       None
   -------------------------------------------------------------------- */

   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: stack.Release()
   *-- Example.....: oMyStack.Release()
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      release object this

   RETURN NULL
   *-- EoM: Release()

   FUNCTION Empty
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/04/1996
   *-- Notes.......: Checks if stack is empty.
   *-- Written for.: Visual dBASE 5.5a
   *-- Rev. History: 11/04/1996 -- Original
   *-- Calls.......: None
   *-- Usage.......: oMyStack.empty()
   *-- Example.....: lError = oMyStack.empty()
   *-- Returns.....: .t. if stack is empty, .f. otherwise
   *-- Parameters..: none
   *-------------------------------------------------------------------
   
   RETURN ( this.size = 0 )
   *-- EoM: Empty()

   FUNCTION Push ( xElement )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/04/1996
   *-- Notes.......: Adds an element to the stack. Stack elements can 
   *--               be of any type.
   *-- Written for.: Visual dBASE
   *-- Rev. History: 11/04/1996 -- Original
   *-- Calls.......: None
   *-- Usage.......: oMyStack.Push( <xElement> )
   *-- Example.....: oMyStack.Push( cWord )
   *-- Returns.....: None
   *-- Parameters..: xElement = element to add to stack
   *-------------------------------------------------------------------

      *-- add a new element
      this.add( xElement )

   RETURN ""
   *-- EoM: Push()

   FUNCTION Pop
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/05/1996
   *-- Notes.......: Remove top element from the stack (if stack not 
   *--                 empty, if it is, it returns NULL).
   *-- Written for.: Visual dBASE
   *-- Rev. History: 11/05/1996 -- Original
   *-- Calls.......: stack::Empty()
   *-- Usage.......: <stackName>.Pop()
   *-- Example.....: xLastElement = oMyStack.Pop()
   *-- Returns.....: Element being removed or NULL if stack is empty
   *--               (You can check for NULL: if oMyStack.pop() = NULL)
   *-- Parameters..: None
   *-------------------------------------------------------------------

      if .not. this.empty()
         xReturn = this[ this.size ]
         this.size = this.size -1 
      else
         xReturn = NULL
      endif

   RETURN xReturn
   *-- EoM: Pop()
   
   FUNCTION GetTop
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/05/1996
   *-- Notes.......: Returns top element from the stack (if stack not 
   *--                 empty) without 'popping' it.
   *-- Written for.: Visual dBASE
   *-- Rev. History: 11/05/1996 -- Original
   *-- Calls.......: stack::Empty()
   *-- Usage.......: <stackName>.GetTop()
   *-- Example.....: xTopElement = oMyStack.GetTop()
   *-- Returns.....: Topmost element in the stack, NULL if stack is 
   *--               empty (You can check for NULL: 
   *--                      if oMyStack.pop() = NULL )
   *-- Parameters..: None
   *-------------------------------------------------------------------
  
   RETURN ( iif( this.size > 0, this[ this.size ], NULL ) )
   *-- EoM: GetTop()
   
ENDCLASS
// -- End of Class: Stack

/* -----------------------------------------------------------------------
   End of File: STACK.CC
   ----------------------------------------------------------------------- */
